/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"

#include "initng_also.h"

s_entry ALSO_START = { "also_start", STRINGS, 10, STD_OPT };
s_entry ALSO_STOP = { "also_stop", STRINGS, 9, STD_OPT };

int service_state(active_h * service)
{
    char *tmp = NULL;
    active_h *current = NULL;

    assert(service);
    assert(service->name);

    switch (service->a_status)
    {

            /*
             * when this service is loading, also start services NEEDed and 
             * marked ALSO_START
             */
        case LOADING:
            tmp = NULL;
            while ((tmp =
                    active_db_get_next_string(&ALSO_START, service, tmp)))
            {
                /* HACK WARNING
                 * service name starting with "virtual/" is no idea to start, sience
                 * they wont exists anyway, this is a waist of resources.
                 */
                if (strncmp(tmp, "virtual/", 8) == 0)
                    continue;

                if (!start_new_service_named(tmp))
                {
                    F_("%s also_start %s could not start!\n", service->name,
                       tmp);
                    stop_service(service);
                    return (FALSE);
                }
            }
            break;



            /*
             * If this service got marked STOPPED, then stop services
             * listed in ALSO_STOP for this service.
             */
        case STOP_MARKED:
        case STOPPED:
            /* Handle ALSO_STOP */
            tmp = NULL;
            while ((tmp =
                    active_db_get_next_string(&ALSO_STOP, service, tmp)))
                if ((current = active_db_find_by_name(tmp)))
                {
                    W_("service %s also stops %s\n", service->name, tmp);
                    stop_service(current);
                }
            break;



            /* default, do nothing */
        default:
            break;
    }
    return (TRUE);
}


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&ALSO_START);
    initng_sdt_add(&ALSO_STOP);
    initng_add_hook(ASTATUS_CHANGE, 50, &service_state);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&ALSO_START);
    initng_sdt_del(&ALSO_STOP);
    initng_del_hook(ASTATUS_CHANGE, &service_state);
}
