
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>
#include <errno.h>


#ifdef BUSYBOX
#include "../../busybox-1.00/include/busybox.h"
#include "../../busybox-1.00/include/libbb.h"
#endif

#include "../../src/initng_active_db.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_signal.h"
#include "../../src/initng_fork.h"

#include "initng_bash_launcher.h"

s_entry START_SCRIPT = { "start{", MULTILINE_STRING, 6, SERVICE_OPT };
s_entry STOP_SCRIPT = { "stop{", MULTILINE_STRING, 5, SERVICE_OPT };
s_entry DAEMON_SCRIPT = { "daemon{", MULTILINE_STRING, 7, DAEMON_OPT };
s_entry KILL_SCRIPT = { "kill{", MULTILINE_STRING, 5, DAEMON_OPT };

static void bash_this(char *bash_code, active_h * s, char *args);
static int bash_exec(process_h * process_to_exec, active_h * s,
                     char *script, char *args);


static int initng_bash(active_h * service, process_h * process)
{
    char *e = NULL;
    s_entry *EXEC = NULL;
    s_entry *ARGS = NULL;


    assert(service);
    assert(service->name);

    /* set variable, depend on type to launch */
    switch (process->pt)
    {
        case T_START:
            EXEC = &START_SCRIPT;
            ARGS = initng_sdt_find("start_args");
            break;
        case T_STOP:
            EXEC = &STOP_SCRIPT;
            ARGS = initng_sdt_find("stop_args");
            break;
        case T_DAEMON:
            EXEC = &DAEMON_SCRIPT;
            ARGS = initng_sdt_find("daemon_args");
            break;
        case T_KILL:
            EXEC = &KILL_SCRIPT;
            ARGS = initng_sdt_find("kill_args");
            break;
        default:
            F_("Unknown type to %i bash_launch!\n", process->pt);
            return (FALSE);
    }


    /* WE ARE EXECUTING A START MULTILINE_STRING */
    if ((e = active_db_get_string(EXEC, service)))
    {
        char *args = NULL;

        if (ARGS)
            args = active_db_get_string(ARGS, service);

        return (bash_exec(process, service, e, args));
    }
    return (FALSE);
}


static void bash_this(char *bash_code, active_h * s, char *args)
{
    D_("bash_this(%s);\n", s->name);
    /* Todo : move bash_this into initng_bash_launcher and parse args properly\n" */
    char **argtmp;
    argtmp = (char **) initng_calloc(8, sizeof(char *));
    argtmp[0] =
        (char *) initng_calloc(1, sizeof(char) * (15 + strlen(s->name)));
    strcpy(argtmp[0], "bash_helper[");
    strcat(argtmp[0], s->name);
    strcat(argtmp[0], "]");
    argtmp[1] = i_strdup("-c");
    argtmp[2] = bash_code;
    argtmp[3] = s->name;
    argtmp[4] = args;
    argtmp[5] = NULL;
    execve("/bin/sh", argtmp, new_environ(s));
    free(argtmp[0]);
    free(argtmp);
    F_("bash_this(): child died!\n EEEEEEEEEEEEEEEEEEEEEEERRRRRRRRRRRROOOOOOOOOORR!\n");
    /* exit 1, why???, to emit signal? */
    initng_free();
    _exit(1);
}                                           /* end fork_and_exec() */



static int bash_exec(process_h * process_to_exec, active_h * s,
                     char *script, char *args)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the real service kicker */
    pid_t pid_fork;             /* pid got from fork() */

    assert(process_to_exec);
    assert(script);

    if ((pid_fork = initng_fork(s, process_to_exec)) == 0)
    {
        /* execute code */
        bash_this(script, s, args);

        /* Bash This should NOT return! */
        _exit(33);
    }

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);

    if (pid_fork > 1)
    {
        process_to_exec->pid = pid_fork;
        return (TRUE);
    }

    process_to_exec->pid = 0;
    return (FALSE);

}

int module_init(const char *version)
{
    D_("initng_simple_plugin: module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&START_SCRIPT);
    initng_sdt_add(&STOP_SCRIPT);
    initng_sdt_add(&DAEMON_SCRIPT);
    initng_sdt_add(&KILL_SCRIPT);

    initng_add_hook(LAUNCH, 51, &initng_bash);
    return (TRUE);
}

void module_unload(void)
{
    D_("initng_simple_plugin: module_unload();\n");
    initng_sdt_del(&START_SCRIPT);
    initng_sdt_del(&STOP_SCRIPT);
    initng_sdt_del(&DAEMON_SCRIPT);
    initng_sdt_del(&KILL_SCRIPT);

    initng_del_hook(LAUNCH, &initng_bash);
}
