/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"

#include "initng_fstat.h"

s_entry FILES_TO_EXIST = { "require_file", STRINGS, 12, STD_OPT };
s_entry FILES_TO_EXIST_AFTER =
    { "files_to_exist_after", STRINGS, 20, STD_OPT };

/* returns TRUE if all use deps are started */
static int check_files_to_exist(active_h * service, e_a_status status)
{
    char *file = NULL;
    struct stat file_stat;

    S_;

    switch (status)
    {
        case START_DEP_MET:
            /* Handle NEED */
            while ((file =
                    active_db_get_next_string(&FILES_TO_EXIST, service,
                                              file)))
            {
                D_("Service %s need file %s to exist\n", service->name, file);
                if (stat(file, &file_stat) != 0)
                {
                    D_("File %s needed by %s doesn't exist.\n", file,
                       service->name);
                    /* set the alarm, make sure initng will search files, in one second */
                    alarm(1);

                    /* don't change status of service to START_DEP_MET */
                    return (FALSE);
                }
            }
            break;

        case RUNNING:
            while ((file =
                    active_db_get_next_string(&FILES_TO_EXIST_AFTER, service,
                                              file)))
            {
                D_("Service %s need file %s before it can be set to RUNNING\n", service->name, file);
                if (stat(file, &file_stat) != 0)
                {
                    D_("File %s needed by %s doesn't exist.\n", file,
                       service->name);
                    /* set the alarm, make sure initng will search files, in one second */
                    alarm(1);

                    /* don't change status of service to START_READY */
                    return (FALSE);
                }
            }
            break;

        default:
            break;
    }

    /* go ahead, change service status */
    return (TRUE);
}


int module_init(const char *version)
{
    S_;
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&FILES_TO_EXIST);
    initng_sdt_add(&FILES_TO_EXIST_AFTER);
    initng_add_hook(CONFIRM_ASTATUS_CHANGE, 55, &check_files_to_exist);
    return (TRUE);
}

void module_unload(void)
{
    S_;
    initng_sdt_del(&FILES_TO_EXIST);
    initng_sdt_del(&FILES_TO_EXIST_AFTER);
    initng_del_hook(CONFIRM_ASTATUS_CHANGE, &check_files_to_exist);
}
