
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>
#include <ctype.h>

#include "../../src/initng_global.h"
#include "../../src/initng_process_db.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_handler.h"
#include "../../src/main.h"
#include "../../src/initng_active_db.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin.h"
#include "../../src/initng_load_module.h"

#include <utmp.h>
#include "initreq.h"

#include "initng_initctl.h"
int utmp_stored = FALSE;

/* static local function here */
static void parse_control_input(void);
static void initctl_control_close(void);
static int initctl_control_open(void);
static void makeutmp(int runlevel);

f_module_h pipe_fd = { &parse_control_input, -1 };      /* /dev/initctl */

struct stat st, st2;

#define PIPE_FD    10                       /* Fileno of initfifo. */

static void initctl_control_close(void)
{
    if (pipe_fd.fds > 2)
    {
        close(pipe_fd.fds);
        pipe_fd.fds = -1;
    }
}

static int initctl_control_open(void)
{
    D_("initctl control open (%d)\n", pipe_fd.fds);
    /* First, try to create /dev/initctl if not present. */
    if (stat(INIT_FIFO, &st2) < 0 && errno == ENOENT)
        (void) mkfifo(INIT_FIFO, 0600);

    /*
     *  If /dev/initctl is open, stat the file to see if it
     *  is still the _same_ inode.
     */
    if (pipe_fd.fds > 2)
    {
        if (fstat(pipe_fd.fds, &st) < 0)
            initctl_control_close();
        if (stat(INIT_FIFO, &st2) < 0 || st.st_dev != st2.st_dev
            || st.st_ino != st2.st_ino)
        {
            initctl_control_close();
        }
    }

    /* If the pipe isn't open, try to open it. */
    if (pipe_fd.fds < 3)
    {
        /* it the filedescriptor has to be over 2 to be valid */
        if ((pipe_fd.fds = open(INIT_FIFO, O_RDWR | O_NONBLOCK)) < 3)
            return (FALSE);

        D_("Opened on fd %i\n", pipe_fd.fds);
        fstat(pipe_fd.fds, &st);
        if (!S_ISFIFO(st.st_mode))
        {
            /* /dev/initctl is there, but we can't open it */
            F_("%s is not a fifo\n", INIT_FIFO);
            if (pipe_fd.fds >= 0)
            {
                close(pipe_fd.fds);
                pipe_fd.fds = -1;
            }
            return (FALSE);
        }

        /* ok, finally add hook */
        initng_add_hook(FDWATCHERS, 70, &pipe_fd);
    }
    return (TRUE);
}

/* To be called when there is input on the control bus */
void parse_control_input(void)
{
    int n;
    struct init_request request;

    /* Read data from /dev/initctl */
    n = read(pipe_fd.fds, &request, sizeof(request));

    /* Check if request is ok : */
    if (n == 0)
    {
        F_("read 0 bytes, this should never happen!\n");
        return;
    }
    if (n <= 0)
    {
        if (errno == EINTR)
            return;
        F_("Error reading request\n");
        return;
    }
    if (request.magic != INIT_MAGIC || n != sizeof(request))
    {
        F_("got bogus initrequest\n");
        return;
    }

    /*
     * Check that the request command is a valid one.
     */
    if (request.cmd != INIT_CMD_RUNLVL)
    {
        D_("got unimplemented initrequest - %d (%c),%d (%c).\n",
           request.runlevel, request.runlevel, request.cmd, request.cmd);
        return;
    }

    /* Todo, handle these:
       #define INIT_CMD_START              0
       #define INIT_CMD_RUNLVL        1
       #define INIT_CMD_POWERFAIL     2
       #define INIT_CMD_POWERFAILNOW  3
       #define INIT_CMD_POWEROK       4
     */

    /* Request is OK, handle it: */
    D_("init data is : - %d (%c),%d (%c).\n", request.runlevel,
       request.runlevel, request.cmd, request.cmd);

    switch (request.runlevel)
    {
        case '0':
            D_("Starting halt service.\n");
            g.when_out = THEN_HALT;
            stop_all();
            return;
        case '6':
            D_("Starting halt service.\n");
            g.when_out = THEN_REBOOT;
            stop_all();
            return;
        default:
            D_("Starting runlevel%c ", request.runlevel);
            {
                char tmp[20];

                sprintf(tmp, "runlevel%c", request.runlevel);
                start_new_service_named(tmp);
            }
            return;
    }
}

static void makeutmp(int runlevel)
{
    D_("Making utmp file for runlevel %d\n", runlevel);
    struct utmp utmp;
    time_t t;

    /*
     * this is created by bootmisc, if this isn't there we can't set runlevel.
     */
    if (access(UTMP_FILE, F_OK) < 0)
    {
        F_("/var/run/utmp does not exist, this should be created by bootmisc.i\n");
        return;
    }
    /*
       todo, is this a good idea or a bad idea?
       utmpname("/var/run/utmp");
     */

    setutent();
    memset(&utmp, 0, sizeof(utmp));
    utmp.ut_type = RUN_LVL;
    utmp.ut_pid = ('#' << 8) + runlevel + '0';
    time(&t);
    utmp.ut_time = (int) t;
    if (pututline(&utmp) == NULL)
    {
        F_("pututline failed\n");
        endutent();
        return;
    }
    endutent();
    return;
}

/* try open fifo, every started service */
int service_status(active_h * service)
{
    if (service->a_status != DONE && service->a_status != RUNNING)
        return (TRUE);

    if (!initctl_control_open())
    {
        F_("Warning, failed to open /dev/initctl\n");
        return (TRUE);
    }
    return (TRUE);
}

void is_system_up(h_sys_state state)
{
    if (state == STATE_UP && (!utmp_stored))
    {
        makeutmp(3);
    }
}


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    if (!g.i_am_init)
    {
        initng_unload_module_named("initctl");
        return (TRUE);                      /* this is not a failure */
    }

    utmp_stored = FALSE;

    initctl_control_open();

    if ((!initng_add_hook(ASTATUS_CHANGE, 50, &service_status))
        || (!initng_add_hook(SWATCHERS, 50, &is_system_up)))
    {
        F_("Fail add hook!\n");
        return (FALSE);
    }
    return (TRUE);
}


void module_unload(void)
{
    D_("module_unload();\n");

    /* then this module was never loaded */
    if (!g.i_am_init)
        return;                             /* this is not a failure */


    initctl_control_close();
    /* remove all hooks */
    initng_del_hook(FDWATCHERS, &pipe_fd);
    initng_del_hook(SWATCHERS, &is_system_up);
    initng_del_hook(ASTATUS_CHANGE, &service_status);
}
