
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <assert.h>

#include "../../src/initng_global.h"
#include "../../src/initng_process_db.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_handler.h"
#include "../../src/main.h"
#include "../../src/initng_active_db.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_load_module.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_global.h"
#include "../../src/initng_error.h"
#include "../../src/initng_string_tools.h"

#include "initng_unneeded.h"


static int cmd_stop_unneeded(char *arg);
s_command STOP_UNNEEDED =
    { 'y', "stop_unneeded", TRUE_OR_FALSE_COMMAND, NO_OPT,
    {(void *) &cmd_stop_unneeded},
    "Stop all services, not in runlevel"
};

static int cmd_stop_unneeded(char *arg)
{

    int needed = FALSE;
    active_h *A, *As = NULL;
    active_h *B = NULL;

    S_;

    /* walk thru all and check */
    while_active_db_safe(A, As)
    {
        /* don't stop virtals */
        if (A->type == TYPE_VIRTUAL)
            continue;
        if (A->type == TYPE_RUNLEVEL)
            continue;

        /* reset variables */
        needed = FALSE;
        B = NULL;

        while_active_db(B)
        {
            /* dont check ourself */
            if (A == B)
                continue;

            /* if B needs A */
            if (active_db_dep_on(B, A) == TRUE)
            {
                needed = TRUE;
                break;
            }
        }

        /* if there was no needed this service, stop it */
        if (needed == FALSE)
            stop_service(A);
    }

    return (TRUE);
}




int module_init(const char *version)
{
    D_("module_init(unneeded);\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_command_add(&STOP_UNNEEDED);

    D_("libunneeded.so.0.0 loaded!\n");
    return (TRUE);
}


void module_unload(void)
{
    D_("module_unload(unneeded);\n");

    initng_command_del(&STOP_UNNEEDED);


    D_("libunneeded.so.0.0 unloaded!\n");

}
