/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#define TRUE 1
#define FALSE 0
#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>

#define DEBUG

#ifdef DEBUG
#define D_(fmt,...) printf(fmt, ## __VA_ARGS__)
#else
#define D_(fmt,...)
#endif

/* this program will parse all .i files in installation
   setting distribution dependent values on install time */

static int is_file(const char *filename)
{
    struct stat tmp;

    D_("Trying, with file \"%s\" : ", filename);
    if (stat(filename, &tmp) == 0 && S_ISREG(tmp.st_mode))
    {
        D_("Found it.\n");
        return (TRUE);
    }
    D_("Did not find it.\n");
    return (FALSE);
}

static const char *probe_distribution(void)
{
    D_("probe_distribution\n");
    if (is_file("/etc/debian_version"))
        return ("debian");
    if (is_file("/etc/gentoo-release"))
        return ("gentoo");
    if (is_file("/etc/fedora-release"))
        return ("fedora");
    if (is_file("/etc/pld-release"))
        return ("pld");
    if (is_file("/etc/pingwinek-release"))
        return ("pingwinek");
    if (is_file("/etc/altlinux-release"))
        return ("altlinux");
    D_("Probe failed\n");
    return ("unknown");
}

static char *do_exec(char *data, int length)
{
    int pipe_fork[2], inpipe_fork[2];

    pipe(pipe_fork);
    pipe(inpipe_fork);

    int status;
    int datalen = 0;
    int pid;
    char *return_data = NULL;

    D_("forking...\n");
    if ((pid = fork()) == 0)
    {
        const char *argv[] = { "/bin/bash", NULL, NULL };
        dup2(pipe_fork[0], 0);
        close(pipe_fork[1]);
        dup2(inpipe_fork[1], 1);
        dup2(inpipe_fork[1], 2);
        close(inpipe_fork[0]);
        execve("/bin/bash", (char **) argv, environ);
        printf(" ** execve /bin/bash FAILED!\n");
        exit(1);
    }
    else
    {
        D_("This is not fork\n");
        return_data = malloc(1024);
        close(pipe_fork[0]);

        /*        printf("Writing data to pipe : (%d) \n",length);
           int x;
           for (x = 0; x <= length;x++)
           printf("%c",data[x]);
           printf("\n"); */

        D_("Data sent to fork (%i): \"%s\"\n", length, data);
        write(pipe_fork[1], data, length);
        write(pipe_fork[1], "\n\nexit\n", 7);
        close(pipe_fork[1]);

        wait(&status);

        int fd_flags;

        fd_flags = fcntl(inpipe_fork[0], F_GETFL, 0);
        fcntl(inpipe_fork[0], F_SETFL, fd_flags | O_NONBLOCK);
        close(inpipe_fork[1]);
        D_("Reading data from fork\n");
        datalen = read(inpipe_fork[0], return_data, 1024);
        close(inpipe_fork[0]);
        if (datalen > 0)
        {
            D_("Read %d bytes: \"%s\"\n", datalen, return_data);
            return return_data;
        }
        printf("Did not got any data from #exec statement!\n");
        free(return_data);
        return NULL;
    }
    return NULL;
}

static int keyword_in_row(const char *key, const char *row)
{
    char *tmp;
    char *st=strndup(row, strcspn(row, "\n\0;"));
    tmp = strstr(st, key);
    printf("checking for %s in %s\n", key, st);
    free(st);
    if(tmp) return(TRUE);
    return(FALSE);
}


int main(int argc, char **argv)
{
    int i = 1;
    char *in_buf = NULL;
    char *out_buf = NULL;
    char *pt = NULL;
    off_t fsize = 0;
    int f_in;
    int f_out;
    int g_e = 0;
    int print_row = 1;
    int exec_row = 0;
    char exec_buffer[1024];
    char *data;
    const char *distro = NULL;
    int distro_len;
    const char *in = NULL;
    const char *out = NULL;


    (void) argc;

    /* check cmdline for set distribution */
    while (argv[i])
    {

        if (argv[i + 1])
        {
            if (strcmp(argv[i], "--dist") == 0 || strcmp(argv[i], "-d") == 0)
            {
                distro = argv[i + 1];
                i++;
            }
            else if (strcmp(argv[i], "--in") == 0
                     || strcmp(argv[i], "-i") == 0)
            {
                in = argv[i + 1];
                i++;
            }
            else if (strcmp(argv[i], "--out") == 0
                     || strcmp(argv[i], "-o") == 0)
            {
                out = argv[i + 1];
                i++;
            }
        }
        i++;
    }

    /* if not set, probe it */
    if (!distro)
        distro = probe_distribution();
    if (!distro)
    {
        fprintf(stderr, "Unknown DISTRIBUTION\n");
        exit(1);
    }
    distro_len = strlen(distro);
    fprintf(stderr, "Distribution set: \"%s\"\n", distro);

    if (!in)
    {
        fprintf(stderr, "Have to set input file with -i\n");
        exit(1);
    }


    f_in = open(in, O_RDONLY);
    if (!f_in)
    {
        fprintf(stderr, "Unable to open input file %s\n", in);
        return (1);
    }

    /* get in size, so we not how mutch to alloc */
    fsize = lseek(f_in, 0, SEEK_END);
    lseek(f_in, 0, SEEK_SET);

    /* allocate in and out buf, we hope twice as much as we are now should be enough. */
    in_buf = calloc((fsize * 2) + 1, sizeof(char));
    out_buf = calloc((fsize * 2) + 11, sizeof(char));

    /* zero outbuf */
    out_buf[0] = '\0';

    /* read input */
    read(f_in, in_buf, fsize);

    /* close input */
    close(f_in);

    /*fprintf(stderr,"File (%s): \"%s\"\n\n\n", in, in_buf); */

    /* set a temporary pointer */
    pt = in_buf;

    /* go tru every row */
    while (pt[0])
    {
        int row_len = 0;

        /* calculate row lenght */
        while (pt[row_len] != '\n' && pt[row_len])
            row_len++;
        if (pt[row_len] == '\n')
            row_len++;

        /* if we are gonna handle this output */
        if (pt[0] == '#')
        {
            /* don't lissen on this commands without print_row on */
            if (print_row)
            {
                if (strncmp("#exec", pt, 5) == 0)
                {
                    D_(" #exec block begins.\n");
                    exec_row = 1;
                    exec_buffer[0] = '\0';
                    goto next;
                }
                else if (strncmp("#endexec", pt, 8) == 0)
                {
                    D_(" #exec block ends.\n");
                    exec_row = 2;
                    goto next;
                }
            }


	    /* this is a if block, if found distro in row set print_row = 1 */
            if (strncmp("#ifd", pt, 4) == 0)
            {
                D_(" #ifd block : ");

		if(keyword_in_row(distro, pt + 5))
		    print_row = 1;
		else
		    print_row = 0;
		    
		g_e = 0;
		
#ifdef DEBUG
                if (print_row == 1)
                    printf("Printing..\n");
                else
                    printf("Quiet..\n");
#endif
                goto next;
            }
	    
	    
            else if (strncmp("#elsed", pt, 6) == 0)
            {
                D_(" #elsed block :");
		if(pt[5]=='\n' || pt[6]=='\n')
		{
		    if(g_e==1)
		    {
			print_row=0;
			goto next;
		    }
	
		    if (print_row==1)
			print_row=0;
		    else if (print_row==0)
			print_row=1;
		    goto next;
		}

		g_e = 1;
		if(keyword_in_row(distro, pt + 6))
		{
		    print_row = 1;
		} else {
		    print_row = 0;
		}

#ifdef DEBUG
                    if (print_row == 1)
                        printf("Printing..\n");
                    else
                        printf("Quiet..\n");
#endif
                    goto next;
            }

            else if (strncmp("#endd", pt, 5) == 0)
            {
                D_("#endd block, all prints.\n");
                /* print as usual */
                print_row = 1;
                goto next;
            }
        }

        /* if exec_row is 1, it is feching data */
        if (exec_row == 1)
        {
            strncat(exec_buffer, pt, row_len);
            /*print_row = 0; */
            goto next;
        }

        /* if exec_row is 2, it is completed fetching data, and its executed */
        if (exec_row == 2)
        {
            exec_row = 0;
            /*printf("Do_exec\n"); */
            data = do_exec(exec_buffer, strlen(exec_buffer));
            D_("exec data is : \"%s\"\n", data);
            if (data)
            {
                strncat(out_buf, data, strlen(data));
                free(data);
            }
        }

        /* else we youst forwards - calculate row lenght */
        if (print_row)
            strncat(out_buf, pt, row_len);

      next:
        /* go to end */
        while (pt[0] != '\n' && pt[0])
            pt++;
        /* go to next row */
        if (pt[0] == '\n')
            pt++;
    }

    /* free input buffer */
    free(in_buf);

    /* open output for writing */
    if (out)
    {
        f_out = open(out, O_WRONLY | O_CREAT | O_TRUNC, 0644);
        /* write output */
        write(f_out, out_buf, strlen(out_buf));
        close(f_out);
    }
    else
    {
        /* if its a .ii file */
        if (strstr(in, ".ii"))
        {
            char *out_2 = NULL;

            /* take away the last i */
            out_2 = strndup(in, strlen(in) - 1);
            f_out = open(out_2, O_WRONLY | O_CREAT | O_TRUNC, 0644);
            /* write output */
            write(f_out, out_buf, strlen(out_buf));
            close(f_out);
        }
        else
        {

            fprintf(stderr, "out with -o not set, defaulting to stdout.\n");
            printf("%s", out_buf);

        }
    }

    /* free output buffer */
    free(out_buf);

    exit(0);
}
